#include <Foundation/Foundation.h>
#include <EOAccess/EOAccess.h>
#include <EOControl/EOControl.h>

int
main(int arcg, char *argv[], char **envp)
{
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
  EOModelGroup *group = [EOModelGroup defaultGroup];
  EOModel *model;
  EOAdaptor *adaptor;
  EOAdaptorContext *context;
  EOAdaptorChannel *channel; 
  EOEditingContext *ec;
  EODatabaseDataSource *authorsDS;
  NSArray *authors;
  NSArray *tables;
  NSArray *entities = nil;
  id author;
  
  model = [group modelNamed:@"library"];
  
  /* Tools don't have resources so we have to add the model manually */ 
  if (!model)
    {
      NSString *path = @"./library.eomodel";
      model = [[EOModel alloc] initWithContentsOfFile: path];
      [group addModel:model];
      entities = [model entities];
      [model release];
    }

  adaptor = [EOAdaptor adaptorWithModel:model];
  context = [adaptor createAdaptorContext];
  channel = [context createAdaptorChannel];

  [channel openChannel];
  tables = [channel describeTableNames];

  /* Create the tables if they don't exist in the database.  */
  if (![tables count])
    {
      NSDictionary *optDict;
      NSArray *stmts;
      Class exprClass;
      NSUInteger i;

      optDict = [NSDictionary dictionaryWithObjectsAndKeys:
                                @"NO", @"EODropTablesKey",
                                @"NO", @"EODropPrimaryKeySupportKey", nil];
      exprClass = [adaptor expressionClass];
      stmts = [exprClass schemaCreationStatementsForEntities: entities
                                                     options: optDict];
      for (i = 0; i < [stmts count]; i++)
        [channel evaluateExpression: [stmts objectAtIndex: i]];
    }

  ec = [[EOEditingContext alloc] init];
  authorsDS 
    = [[EODatabaseDataSource alloc] initWithEditingContext: ec
				    entityName:@"authors"];

  /* Create a new author object */
  author = [authorsDS createObject];
  [author takeValue:@"Anonymous" forKey:@"name"];
  [authorsDS insertObject:author];
  [ec saveChanges];

  
  /* Fetch the newly inserted object from the database */
  authors = [authorsDS fetchObjects];
  NSLog(@"%@", authors);

  /* Update the authors name */
  [[authors objectAtIndex:0] 
  	takeValue:@"John Doe" forKey:@"name"];
  [ec saveChanges];
 
  NSLog(@"%@", [authorsDS fetchObjects]);

  [channel closeChannel];
  [pool release];
  return 0;
}

